#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Sep 19 16:16:54 2022


Solution to Exercise 1a 


@author: richter
"""

import numpy as np
import matplotlib.pyplot as plt
import torch


## This is the function that we are going to approximate
def f(x):
    return np.log(1+x)*np.exp(np.sin(x))

## We create 20 random training points in [0,10]
Ntrain = 20
x = np.random.rand(Ntrain,1).astype(np.float32) * 10

# we evaluate the function in these points
y = f(x)

## for trainng, we transform the array to a torch tensor
xtrain = torch.from_numpy(x)
ytrain = torch.from_numpy(y)

## We create the model
model = torch.nn.Sequential(
                torch.nn.Linear(1,5),  # 1 input -> 2
                torch.nn.Sigmoid(),    # sigmoid-activation
                torch.nn.Linear(5,1)   # 2 -> 1
                )                      # no further activation

## We specify the parameters for the optimizer
learningrate, maxiter =0.001,20000
loss_fn = torch.nn.MSELoss(reduction='sum')
optimizer = torch.optim.Adam(model.parameters(),lr=learningrate,weight_decay=0)



for t in range(maxiter+1):   # optimization loop
    torch_prediction = model(xtrain)        # evaluate the model
    loss = loss_fn(torch_prediction,ytrain) # compute the loss
    if t%1000==0:                     # output
        print(t,loss.item(),sep='\t')
    optimizer.zero_grad()                   # 
    loss.backward()                         # compute the gradient
    optimizer.step()                        # one step


## We evaluate the model in the training points
ytrain = model(xtrain)

## Also, we evaluate the model along the interval to plot it is a function
x = np.linspace(0,10,100).astype(np.float32)
ymodel = model(torch.from_numpy(np.transpose(np.array([x]))))

## not we plot everyting
plt.plot(xtrain.detach().numpy(),ytrain.detach().numpy(),'*',label='Training points')
plt.plot(x,f(x),label='f(x)')
plt.plot(x,ymodel.detach().numpy(),label='neural network')


plt.legend()
plt.show()
