#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Sep 19 17:25:37 2022

exercise 2. solving an ode

@author: richter
"""

import numpy as np
import matplotlib.pyplot as plt


## The right hand side of the ODE:  x'(t) = A x(t)
def A(x):
    return np.array([-10.0 * x[0]+10.0*x[1],
                     28.0*x[0]-x[1]-x[0]*x[2],
                     x[0]*x[1]-8./3.*x[2]])

## One step of Heun's 2nd order method
def heun(x,h):
    return x+0.5*h*A(x) + 0.5*h*A(x+h*A(x))

## The time discretization
T = 10    # Final time
NT = 1000  # Number of time steps
h = T/NT  # step size

timesteps = np.linspace(0,T,NT+1)   # time mesh

x = np.zeros( (NT+1,3) )      # for storing the solution
x[0,:] = np.array([-10,-10,20])  # the initial value

for n in range(NT):       # time loop
    x[n+1,:] = heun(x[n,:],h)  # 
    
    
## make a 3d figure    
fig = plt.figure()
ax = fig.add_subplot(111, projection='3d')
ax.scatter(x[:,0],x[:,1],x[:,2],linewidth=1,label='(x(t),y(t),z(t))')
plt.xlabel('x')
plt.ylabel('y')

plt.legend()
plt.show()

plt.plot(timesteps,x[:,0],label='x1')
plt.plot(timesteps,x[:,1],label='x2')
plt.plot(timesteps,x[:,2],label='x2')
plt.xlabel('x')
plt.legend()
plt.show()
