#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Sep 19 17:25:37 2022

exercise 2. solving an ode

@author: richter
"""

import numpy as np
import matplotlib.pyplot as plt

## The right hand side of the ODE:  x'(t) = A x(t)
def A(x):
    return np.array([-10.0 * x[0]+10.0*x[1],
                     28.0*x[0]-x[1]-x[0]*x[2],
                     x[0]*x[1]-8./3.*x[2]])

## One step of Heun's 2nd order method
def heun(x,h):
    return x+0.5*h*A(x) + 0.5*h*A(x+h*A(x))

## Create random training data
Ntrain = 10
# random points in [a,b] are generated as a + (b-a) * r 
# where r is in [0,1]
# We use numpy's way of combining vectors (the interval bounds) and tensors 
# (the random values)
x = (np.array([-15,-20,10])+np.array([30,40,30])*np.random.rand(Ntrain,3)).astype(np.float32)  # Make sure that it is float!
y = np.zeros((Ntrain,3))  # for storing the y-value of the data points


## For each of the initial values we compute the solution
## to Lorenz' system for t=0.1
Ttrain = 0.1
for n in range(Ntrain):
    # the solution is computed with Heun's method using hheun=0.01
    hheun = 0.01
    # We hence have to do Ttrain / hheun steps
    Nheun = int(Ttrain/hheun+1.e-8)  # make sure that we round up!!
    
    xheun = np.zeros( (Nheun+1,3) ) # for storing the solution
    xheun[0,:] = x[n,:]             # setting the initial

    for i in range(Nheun):          # do the Heun steps
        xheun[i+1,:] = heun(xheun[i,:],hheun)
    
    y[n,:] = xheun[-1,:]            # store the final point

    plt.title('x = {0:2.4f},{1:2.4f},{2:2.4f}'.format(x[n,0],x[n,1],x[n,2]))
    plt.plot(np.linspace(0,Ttrain,Nheun+1),xheun,label='Heun-solution')
    plt.plot([0],x[n:n+1,:],'*b')
    plt.plot([Ttrain],y[n:n+1,:],'*b')
    plt.legend()
    plt.show()
